unit ThreadControllerUnit;

interface

uses
  Classes, SysUtils, Windows, Messages;

type
  TThreadController = class
  private
    FSyncShutDown: TMultiReadExclusiveWriteSynchronizer;
    FSyncThreads: TMultiReadExclusiveWriteSynchronizer;
    FShutDown: Boolean;
    FThreads: TList;
    procedure SetShutdown(const Value: Boolean);
    function GetShutdown: Boolean;
    function GetActiveThreads: Integer;
    procedure DoShutDown;
  public
    constructor Create;
    destructor Destroy; override;
    procedure AddThread(Thread: TThread);
    procedure RemoveThread(Thread: TThread);
    property Shutdown: Boolean read GetShutdown write SetShutdown;
    property ActiveThreads: Integer read GetActiveThreads;
  end;

implementation

{ TThreadController }

procedure TThreadController.AddThread(Thread: TThread);
begin
  FSyncThreads.BeginWrite;
  try
    if FThreads.IndexOf(Thread) < 0 then
      FThreads.Add(Thread);
  finally
    FSyncThreads.EndWrite;
  end;
end;

constructor TThreadController.Create;
begin
  inherited;
  FSyncShutDown := TMultiReadExclusiveWriteSynchronizer.Create;
  FSyncThreads := TMultiReadExclusiveWriteSynchronizer.Create;
  FShutDown := False;
  FThreads := TList.Create;
  end;

destructor TThreadController.Destroy;
begin
  FSyncShutDown.Free;
  FSyncThreads.Free;
  FThreads.Free;
  inherited;
end;

procedure TThreadController.DoShutDown;
var
 i: integer;
begin
  FSyncThreads.BeginRead;
  try
    for i := 0 to FThreads.Count -1 do
      PostThreadMessage(TThread(FThreads[i]).ThreadId,WM_QUIT,0,0);
    finally
      FSyncThreads.EndRead;
    end;
end;

function TThreadController.GetActiveThreads: Integer;
begin
  FSyncThreads.BeginRead;
  try
    Result := FThreads.Count;
  finally
    FSyncThreads.EndRead;
  end;
end;

function TThreadController.GetShutdown: Boolean;
begin
  FSyncShutDown.BeginRead;
  try
    Result := FShutDown;
  finally
    FSyncShutDown.EndRead;
  end;
end;



procedure TThreadController.RemoveThread(Thread: TThread);
var
  i: integer;
begin
  FSyncThreads.BeginWrite;
  try
    i :=FThreads.IndexOf(Thread);
    if i > -1 then
      FThreads.Delete(i);
  finally
    FSyncThreads.EndWrite;
  end;
end;

procedure TThreadController.SetShutdown(const Value: Boolean);
begin
  FSyncShutDown.BeginWrite;
  try
    FShutdown := Value;
    if FShutDown then
      DoShutDown;
  finally
    FSyncShutDown.EndWrite;
  end;

end;

end.
